/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.tracer.impl;

// =============================================================================


import jatva.tracer.TraceWriter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Properties;

// =============================================================================

/**
 * This class implements multiplexor that allows writing trace to several trace
 * writers simultaneousely.
 * 
 * @author <a href="mailto:barancev@ispras.ru">Alexei Barantsev</a>
 * @version 1.0
 */
public final class WriteToMultiple implements TraceWriter
{

    /**
     * Holds list of trace writers managed by this multiplexor.
     */
    private List<TraceWriter> traceWriters = new ArrayList<TraceWriter>();

    /**
     * Creates a new instance of WriteToMultiple.
     */
    public WriteToMultiple()
    {}

    public WriteToMultiple clone()
    {
        WriteToMultiple newObj = new WriteToMultiple();

        for( TraceWriter tw : getTraceWriters() )
        {
            newObj.addTraceWriter( (TraceWriter)tw.clone() );
        }

        return newObj;
    }

    public void configure( Properties writerProperties )
    {
        for( TraceWriter tw : traceWriters )
        {
            tw.configure( writerProperties );
        }
    }

    /**
     * Adds the given trace writer object to the list of trace writers managed
     * by this multiplexor.
     * 
     * @param traceWriter
     *            Trace writer to be added.
     */
    public void addTraceWriter( TraceWriter traceWriter )
    {
        traceWriters.add( traceWriter );
    }

    /**
     * Gets the list of trace writer objects managed by this multiplexor.
     * 
     * @return Collection of objects that implement <code>TraceWriter</code>
     *         interface.
     */
    public Collection<TraceWriter> getTraceWriters()
    {
        return traceWriters;
    }

    /**
     * Multiplex to all participants.
     * 
     * @see AbstractWriter#setTestId(String)
     */
    public void setTestId( int testId )
    {
        for( TraceWriter tw : traceWriters )
        {
            tw.setTestId( testId );
        }
    }

    /**
     * Multiplex to all participants.
     * 
     * @see AbstractWriter#setTraceHeader(StringBuffer)
     */
    public void setTraceHeader( StringBuffer header )
    {
        for( TraceWriter tw : traceWriters )
        {
            tw.setTraceHeader( header );
        }
    }

    /**
     * Multiplex to all participants.
     * 
     * @see AbstractWriter#setTraceFooter(StringBuffer)
     */
    public void setTraceFooter( StringBuffer footer )
    {
        for( TraceWriter tw : traceWriters )
        {
            tw.setTraceFooter( footer );
        }
    }

    public void prepare()
    {
        for( TraceWriter tw : traceWriters )
        {
            tw.prepare();
        }
    }

    /**
     * Multiplex to all participants.
     * 
     * @see AbstractWriter#sendMessage(StringBuffer)
     */
    public void sendMessage( StringBuffer message )
    {
        for( TraceWriter tw : traceWriters )
        {
            tw.sendMessage( message );
        }
    }

    /**
     * Multiplex to all participants.
     * 
     * @see AbstractWriter#finish()
     */
    public void finish()
    {
        for( TraceWriter tw : traceWriters )
        {
            tw.finish();
        }
    }

}
// =============================================================================
// End of file WriteToMultiple.java
